/*********************************************************************
 *                                                                   *
 *                M65816: Portable 65816 CPU Emulator                *
 *                                                                   *
 *        Written and Copyright (C)1996 by Joshua M. Thompson        *
 *                                                                   *
 *  You are free to distribute this code for non-commercial purposes *
 * I ask only that you notify me of any changes you make to the code *
 *     Commercial use is prohibited without my written permission    *
 *                                                                   *
 *********************************************************************/

#define M65816_DISPATCH

#include <stdio.h>
#include "m65816.h"
#include "micros.h"

dualw	A;	/* Accumulator               */
dualw	D;	/* Direct Page Register      */
byte	P;	/* Processor Status Register */
int	E;	/* Emulation Mode Flag       */
dualw	S;	/* Stack Pointer             */
dualw	X;	/* X Index Register          */
dualw	Y;	/* Y Index Register          */
byte	DB;	/* Data Bank Register        */

union {
#ifdef LSB_FIRST
	struct { byte L,H,PB,Z; } B;
	struct { word PC,Z; } W;
#else
	struct { byte Z,PB,H,L; } B;
	struct { word Z,PC; } W;
#endif
	address	A;
} PC;

extern int	RESET,ABORT,NMI,IRQ,STOP,WAIT;
int		uperiod = 10000;
int		ucnt;

extern void	*opcode_table[1300];

void M65816_run(void)
{
#include "table.h"
	byte	opcode;
	int	opcode_offset;
	duala	atmp,opaddr;
	dualw	wtmp,otmp,operand;

	opcode_offset = 1040;
	ucnt = 0;

dispatch:
	if (ucnt++ == uperiod) {
		ucnt = 0;
		E_UPDATE();
	}
	if (RESET) goto *opcode_table[256+opcode_offset];
	if (STOP) goto dispatch;
	if (ABORT) goto *opcode_table[257+opcode_offset];
	if (NMI) goto *opcode_table[258+opcode_offset];
	if (IRQ) goto *opcode_table[259+opcode_offset];
	if (WAIT) goto dispatch;
#ifdef DEBUG
	M65816_Debug();
#endif
	opcode = M_READ(PC.A);
	PC.W.PC++;
	goto *opcode_table[opcode+opcode_offset];

modeSwitch:
	if (E) {
		P |= 0x30;
		S.B.H = 1;
		D.W = 0;
	}
	if (F_getX) {
		X.B.H = 0;
		Y.B.H = 0;
	}
	opcode_offset = ((E << 2) | ((~P >> 4) & 0x03))*260;
	goto dispatch;

#include "opcodes1.h"
#include "opcodes2.h"
#include "opcodes3.h"
#include "opcodes4.h"
#include "opcodes5.h"
}
