extern int	mem_ramsize;

/* Flags that make up the Shadow Register ($C035). */

extern int	mem_shadow_text;
extern int	mem_shadow_hires1;
extern int	mem_shadow_hires2;
extern int	mem_shadow_super;
extern int	mem_shadow_aux;
extern int	mem_shadow_lc;

/* The slot control bits in the slot register */

extern int mem_slot_reg[8];

/* Language card control registers */

extern int	mem_lcbank2;
extern int	mem_lcread;
extern int	mem_lcwrite;
extern int	mem_lcsecond;

/* Auxillary memory access registers */

extern int	mem_auxrd;
extern int	mem_auxwrt;
extern int	mem_altzp;
extern int	mem_80store;

/* Diagnostics and Interrupt status register */

extern int	mem_diagtype;

/* Master pointers to all 256 possible 64k banks.	*/
/* Nonexistant banks are marked by NULL.		*/

extern byte	*memory[256];

/* Structure used to handle reads and writes in	*/
/* the "special" memory in banks $00, $01, $E0,	*/
/* and $E1. This structure handles one page of	*/
/* a bank. If the read or write pointer is NULL	*/
/* then the request is forwarded to one of the	*/
/* fault handlers.				*/

typedef struct {
	byte	*readPtr;
	byte	*writePtr;
	address	readOffset;
	address	writeOffset;
	byte	(*readFaultHandler)(address);
	void	(*writeFaultHandler)(address,byte);
} mem_pagestruct;

extern mem_pagestruct	bank00[256];
extern mem_pagestruct	bank01[256];
extern mem_pagestruct	bankE0[256];
extern mem_pagestruct	bankE1[256];

extern int	MEM_init(void);
extern void	MEM_update(void);
extern void	MEM_reset(void);
extern void	MEM_shutdown(void);

extern void	MEM_buildMap(void);
extern void	MEM_buildLanguageCard(int, mem_pagestruct *);

extern byte	MEM_readMem(address);
extern void	MEM_writeMem(address, byte);

extern byte	MEM_defaultReadFaultHandler(address);
extern void	MEM_defaultWriteFaultHandler(address,byte);
extern byte	MEM_ioReadFaultHandler(address);
extern void	MEM_ioWriteFaultHandler(address,byte);

extern byte	MEM_clear80store(byte val);
extern byte	MEM_set80store(byte val);
extern byte	MEM_get80store(byte val);
extern byte	MEM_clearAuxRd(byte val);
extern byte	MEM_setAuxRd(byte val);
extern byte	MEM_getAuxRd(byte val);
extern byte	MEM_clearAuxWrt(byte val);
extern byte	MEM_setAuxWrt(byte val);
extern byte	MEM_getAuxWrt(byte val);
extern byte	MEM_clearAltZP(byte val);
extern byte	MEM_setAltZP(byte val);
extern byte	MEM_getAltZP(byte val);
extern byte	MEM_getLCbank(byte val);
extern byte	MEM_getLCread(byte val);
extern byte	MEM_getSlotReg(byte val);
extern byte	MEM_setSlotReg(byte val);
extern byte	MEM_getShadowReg(byte val);
extern byte	MEM_setShadowReg(byte val);
extern byte	MEM_getIntEnReg(byte val);
extern byte	MEM_setIntEnReg(byte val);
extern byte	MEM_getDiagType(byte val);
extern byte	MEM_setVBLClear(byte val);
extern byte	MEM_getStateReg(byte val);
extern byte	MEM_setStateReg(byte val);
extern byte	MEM_setLCx80(byte val);
extern byte	MEM_setLCx81(byte val);
extern byte	MEM_setLCx82(byte val);
extern byte	MEM_setLCx83(byte val);
extern byte	MEM_setLCx88(byte val);
extern byte	MEM_setLCx89(byte val);
extern byte	MEM_setLCx8A(byte val);
extern byte	MEM_setLCx8B(byte val);

extern byte	MEM_getC071(byte val);
extern byte	MEM_getC072(byte val);
extern byte	MEM_getC073(byte val);
extern byte	MEM_getC074(byte val);
extern byte	MEM_getC075(byte val);
extern byte	MEM_getC076(byte val);
extern byte	MEM_getC077(byte val);
extern byte	MEM_getC078(byte val);
extern byte	MEM_getC079(byte val);
extern byte	MEM_getC07A(byte val);
extern byte	MEM_getC07B(byte val);
extern byte	MEM_getC07C(byte val);
extern byte	MEM_getC07D(byte val);
extern byte	MEM_getC07E(byte val);
extern byte	MEM_getC07F(byte val);
