/*********************************************************************
 *                                                                   *
 *                     XGS : Apple IIGS Emulator                     *
 *                                                                   *
 *        Written and Copyright (C)1996 by Joshua M. Thompson        *
 *                                                                   *
 *  You are free to distribute this code for non-commercial purposes *
 * I ask only that you notify me of any changes you make to the code *
 *     Commercial use is prohibited without my written permission    *
 *                                                                   *
 *********************************************************************/

/*
 * File: rw.c
 *
 * This is the top-level code for reading and writing bytes from
 * emulator RAM, calling the appropriate fault routines for accesses
 * to special memory areas.
 */

#include <xgs.h>
#include "memory.h"

byte MEM_readMem(address addr)
{
	int	bank,page,offset;
	byte	*base;

	addr &= 0xFFFFFF;
#ifdef HACK_TO_MAKE_GSOS_SORTA_LOAD
	if (addr == 0x01FAE6) return 0xF0;
	if (addr == 0x01F4DD) return 0xD0;
#endif
	bank = (addr >> 16);
	offset = addr & 0xFFFF;
	page = offset >> 8;
	switch(bank) {
		case 0x00 :	base = bank00[page].readPtr;
				if (!base) return (*bank00[page].readFaultHandler)(addr+bank00[page].readOffset);
				offset &= 0xFF;
				break;
		case 0x01 :	base = bank01[page].readPtr;
				if (!base) return (*bank01[page].readFaultHandler)(addr+bank01[page].readOffset);
				offset &= 0xFF;
				break;
		case 0xE0 :	base = bankE0[page].readPtr;
				if (!base) return (*bankE0[page].readFaultHandler)(addr+bankE0[page].readOffset);
				offset &= 0xFF;
				break;
		case 0xE1 :	base = bankE1[page].readPtr;
				if (!base) return (*bankE1[page].readFaultHandler)(addr+bankE1[page].readOffset);
				offset &= 0xFF;
				break;
		default :	base = memory[bank];
				if (!base) return 0;
				break;
	}
	return *(base+offset);
}

void MEM_writeMem(address addr, byte val)
{
	int	bank,page,offset;
	byte	*base;

	addr &= 0xFFFFFF;
	bank = (addr >> 16);
	if (bank >= 0xF0) return;	/* trap writes to ROM */
	offset = addr & 0xFFFF;
	page = offset >> 8;
	switch(bank) {
		case 0x00 :	base = bank00[page].writePtr;
				if (!base) {
					(*bank00[page].writeFaultHandler)(addr+bank00[page].writeOffset,val);
					return;
				}
				offset &= 0xFF;
				break;
		case 0x01 :	base = bank01[page].writePtr;
				if (!base) {
					(*bank01[page].writeFaultHandler)(addr+bank01[page].writeOffset,val);
					return;
				}
				offset &= 0xFF;
				break;
		case 0xE0 :	base = bankE0[page].writePtr;
				if (!base) {
					(*bankE0[page].writeFaultHandler)(addr+bankE0[page].writeOffset,val);
					return;
				}
				offset &= 0xFF;
				break;
		case 0xE1 :	base = bankE1[page].writePtr;
				if (!base) {
					(*bankE1[page].writeFaultHandler)(addr+bankE1[page].writeOffset,val);
					return;
				}
				offset &= 0xFF;
				break;
		default :	base = memory[bank];
				if (!base) return;
				break;
	}
	*(base+offset) = val;
}
